# Example that shows how to make jets from PFA (reconstructed particles) using  LCIO format
# This example makes invariant mass of 2 reconstructed jets and makes Z peak from Z-> tau tau
# This is a simple example that uses Durham algorithm to force exactly 2 jets
# Use Z to bbar data 
#  Get 4 file in 2 thread (see HepSim manual):
#  > hs-get gev250ee_pythia6_zpole_tautau%rfull001 gev250ee_pythia6_zpole_tautau 2 50 
#
# author: S.Chekanov (ANL)


from org.lcsim.event import *
from org.lcsim.util import *
from hep.physics.vec import BasicHepLorentzVector,HepLorentzVector 
from java.util import *
from java.io import *
from org.lcsim.lcio import LCIOReader
from hep.io.sio import SIOReader
from hep.lcio.implementation.sio import SIOLCReader
from hep.lcio.implementation.io import LCFactory
from hep.lcio.event import *
from hep.lcio.io import *
from jhplot import *  # import graphics
from hephysics.particle import LParticle 
from hep.physics.jet import FixNumberOfJetsFinder
from math import *

# make list of files..
import glob
files=glob.glob("gev250ee_pythia6_zpole_tautau/*.slcio")
factory = LCFactory.getInstance()
reader = factory.createLCReader()
reader.open(files)

h1=H1D("M(jj)",30,30,120)
# h1.doc()  # look at API

# jet description
# http://java.freehep.org/freehep-physics/apidocs/hep/physics/jet/package-summary.html
# this is Jade
from hep.physics.jet import DurhamJetFinder,FixNumberOfJetsFinder
# fjet=FixNumberOfJetsFinder(2) # request 2 jets. The Jade algorithm at work 
fjet=DurhamJetFinder(0.05) 
nEvent=0
while(1):
     evt=reader.readNextEvent()
     if (evt == None): break
     nEvent=nEvent+1
     # print " file event: ",evt.getEventNumber(), " run=",evt.getRunNumber()
     if (nEvent%100==0): print "# Event: ",nEvent
     strVec = evt.getCollectionNames()
     if nEvent == 1:
            for col in  strVec:
                           print col           
     col = evt.getCollection("PandoraPFOCollection")
     nPFA = col.getNumberOfElements()

     particles=ArrayList() # list of particles  
     for i in range(nPFA):
          # http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/lcio/SIOReconstructedParticle.html
          # http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/event/ReconstructedParticle.html
          pa=col.getElementAt(i)
          charge=pa.getCharge()
          p4=pa.getMomentum()  
          ee=pa.getEnergy() 
          typep=pa.getType() # type of PF 
          p=BasicHepLorentzVector(ee,p4[0],p4[1],p4[2])
          particles.add(p) # add particle to the list 
     #print particles.size()
     if (particles.size()>1): # ask for >1 particles
       fjet.setEvent(particles)
       # print "Nr of jets=", fjet.njets()  
       alljets=[] # make a new list with jets 
       for i in range(fjet.njets()):
         # print "Jet=",i," Nr of particles in jet =",fjet.nParticlesPerJet(i) 
         pjet=fjet.jet(i)   # make  HepLorentzVector
         ee=pjet.t()  # magnitude
         p3=pjet.v3() # 3-vector  
         #print " ",ee,p3
         pl=LParticle(p3.x(),p3.y(),p3.z(),ee) # convert to a class with convinient kinematic methods  
         if (pl.perp()>20): alljets.append(pl)  
       if (len(alljets)>1): # calculate invariant mass of two jets 
         for i1 in range(0,len(alljets)-1):
           p1=alljets[i1]
           phi1=p1.phi()
           for i2 in range(i1+1,len(alljets)):
              p2=alljets[i2]
              phi2=p2.phi()
              p1.add(p2)
              if (abs(phi1-phi2)>1.5):
                mass=p1.calcMass()
                h1.fill(mass)


reader.close();

c1=HPlot()
# c1.doc() # look at API
c1.visible()
c1.setMarginLeft(100)
c1.setNameX("M(jj) [GeV]")
c1.setNameY("Events")
c1.setRangeX(30,120)
c1.draw(h1)
c1.export("mc_tautau.pdf") 
 
