# Example that shows how to analyse PFA (reconstructed particles) using  LCIO format
# This example makes invariant mass of 2 reconstructed particles  and makes Z peak 
#  Get 4 files in 2 threads as (see HepSim manual):
#  > hs-get gev250ee_pythia6_zpole_ee%rfull001 gev250ee_pythia6_zpole_ee 2 4  
# S.Chekanov (ANL)

from org.lcsim.event import *
from org.lcsim.util import *
from hep.physics.vec import *
from java.util import *
from java.io import *
from org.lcsim.lcio import LCIOReader
from hep.io.sio import SIOReader
from hep.lcio.implementation.sio import SIOLCReader
from hep.lcio.implementation.io import LCFactory
from hep.lcio.event import *
from hep.lcio.io import *
from jhplot import *  # import graphics
from org.lcsim.event import Track
from org.lcsim.event.base import BaseTrackState
from hephysics.particle import LParticle 
from math import *
from java.lang import System


# make list of files..
import glob
files=glob.glob("gev250ee_pythia6_zpole_ee/*.slcio")
factory = LCFactory.getInstance()

h1=H1D("Mass",70,50,120)
nEvent=0
for f in files:
  print "Open file=",f
  reader = factory.createLCReader()
  reader.open(f)
  while(1):
     evt=reader.readNextEvent()
     if (evt == None): break
     nEvent=nEvent+1
     # print " file event: ",evt.getEventNumber(), " run=",evt.getRunNumber()
     if (nEvent%100==0): print "# Event: ",nEvent
     strVec = evt.getCollectionNames()
     if nEvent == 1:
            for col in  strVec:
                           print col           
     col = evt.getCollection("Tracks")
     ntracks = col.getNumberOfElements()
     Bfield=5.0 # B-field of sidloi3 detector 
     # print "Nr of tracks=",ntracks
     #See: http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/lcio/SIOTrack.html
     particles=[]
     for i in range(ntracks):
          track=col.getElementAt(i)
          trk=BaseTrackState()
          trk.setZ0(track.getZ0())
          trk.setPhi(track.getPhi())
          trk.setOmega(track.getOmega())
          trk.setD0(track.getD0())
          trk.setTanLambda(track.getTanLambda())
          charge=1
          if (track.getOmega()<0): charge=-1
          mom=trk.computeMomentum(Bfield) # B filed in Z 
          px=mom[0]
          py=mom[1]
          pz=mom[2]
          ee=sqrt(px*px+py*py+pz*pz)
          p=LParticle(px,py,pz,ee)
          p.setCharge(charge)
          #print p.perp(), p.rapidity()
          particles.append(p)
     if (len(particles)>1):
       for i1 in range(0,len(particles)-1):
          p1=particles[i1]
          charge1=p1.getCharge()
          for i2 in range(i1+1,len(particles)):
              p2=particles[i2]
              charge2=p2.getCharge()
              p1.add(p2)
              if (charge1*charge2<0):
                  # print p1.calcMass()
                  mass=p1.calcMass()
                  h1.fill(mass)

     del col
     del evt
  reader.close() # close the file
  del reader
  System.gc() # if you low on memory, manual JVM cleanup for every 5th file 


c1=HPlot()
c1.visible()
c1.setAutoRange()
c1.setMarginLeft(100)
c1.setNameX("Mass [GeV]")
c1.setNameY("Events")
c1.draw(h1)
c1.export("mc_tracks.pdf") 
 
