# Example: read truth-level particles and create Z peak  
# Use http://atlaswww.hep.anl.gov/hepsim/show.php?item=146&reco=rfull001 
#  Get 4 files in 2 threads (see HepSim manual):
#  > hs-get gev250ee_pythia6_zpole_ee%rfull001 gev250ee_pythia6_zpole_ee 2 4 
# S.Chekanov (ANL)


from org.lcsim.lcio import LCIOReader
from hep.io.sio import SIOReader
from hep.lcio.implementation.sio import SIOLCReader
from hep.lcio.implementation.io import LCFactory
from hep.lcio.event import * 
from hep.lcio.io import *
from jhplot import *  # import graphics
from hephysics.particle import LParticle
from java.lang import System

# make list of files..
import glob
files=glob.glob("gev250ee_pythia6_zpole_ee/*.slcio")
factory = LCFactory.getInstance()

nEvent=0
h1=H1D("Mass",70,50,120)
for f in files:
  print "Open file=",f
  reader = factory.createLCReader()
  reader.open(f) 
  while(1):
    evt=reader.readNextEvent()
    if (evt == None): break
    nEvent=nEvent+1
    # print " file event: ",evt.getEventNumber(), " run=",evt.getRunNumber()
    if (nEvent%100==0): print "# Event: ",nEvent
    col = evt.getCollection("MCParticle")
    nMc=col.getNumberOfElements()
    particles=[]
    pp=0
    nn=0
    for i in range(nMc): # loop over all particles 
      par=col.getElementAt(i)
      if(par.getGeneratorStatus() == 1 and par.getCharge() !=0): 
              vertex = par.getVertex();
              pdg=par.getPDG()
              momentum = par.getMomentum() 
              ee=par.getEnergy()
              mass=par.getMass()
              px=momentum[0]
              py=momentum[1]
              pz=momentum[2]
              charge=par.getCharge()
              p=LParticle(px,py,pz,ee,mass) 
              p.setCharge(charge)
              particles.append(p)
              #print nn, "pdg=",pdg, " ",px,py,pz, " charge=",charge 
              nn=nn+1
    if (len(particles)>1):
       for i1 in range(0,len(particles)-1):
          p1=particles[i1]
          charge1=p1.getCharge() 
          for i2 in range(i1+1,len(particles)):
              p2=particles[i2]
              charge2=p2.getCharge()
              p1.add(p2)
              # print p1.calcMass(),charge1,charge2 
              if (charge1*charge2<0):
                  mass=p1.calcMass()
                  h1.fill(mass)

    del col
    del evt 
  reader.close() # close the file
  del reader
  System.gc() # if you low on memory, manual JVM cleanup for every 5th file 

c1=HPlot()
c1.visible()
c1.setMarginLeft(100)
c1.setNameX("Mass [GeV]")
c1.setNameY("Events")
c1.setRangeX(50,120)
c1.draw(h1)
c1.export("mc_truth.pdf")

