#!/bin/bash
# Setup file for FPad. It also optimizes the  Fpad installation
# https://atlaswww.hep.anl.gov/asc/jas4pp/ 
# S.Chekanov (ANL)

CFILE="./fpad"
export JAS4PP=`pwd`
if [ ! -f $CFILE ];
then
   export JAS4PP="$( cd "$( echo "${BASH_SOURCE[0]%/*}" )" && pwd )"
fi


CACHE="${JAS4PP}/lib/jython/cachedir/packages"
if [ -d $CACHE ];
then

# setup hepsim
source ${JAS4PP}/lib/physics/setup.sh
# setup Jas4PP
export PATH=${JAS4PP}:${PATH}


# this is for first-time installation
else


if which java >/dev/null; then
    _java=java
elif [[ -n "$JAVA_HOME" ]] && [[ -x "$JAVA_HOME/bin/java" ]];  then
    _java="$JAVA_HOME/bin/java"
else
    echo "No java detected! Please install it from https://java.com/download"; exit 0
fi

if [[ "$_java" ]]; then
    version=$("$_java" -version 2>&1 | awk -F '"' '/version/ {print $2}')
    if [[ "$version" < "1.7" ]]; then
        echo "Java version is less than 1.7. Please update it."; exit 0;
    fi
fi

echo "Setup Jas4pp from $JAS4PP"
echo "Wait.. first-time optimization of Java packages for Jython"
chmod -R 755  "${JAS4PP}/lib/jython/cachedir" 
${JAS4PP}/fpad ${JAS4PP}/lib/jython/optimize.py
export PATH=${PATH}:${JAS4PP}
source ${JAS4PP}/lib/physics/setup.sh


fi
